// --------------------
// Sequelize hierarchy
// Errors
// --------------------

// modules
var _ = require('lodash'),
	util = require('util'),
	semverSelect = require('semver-select');

// exports
module.exports = function(Sequelize) {
	/*
	 * For Sequelize v2 + v3, use `util.inherits` to subclass Error.
	 * For Sequelize v4, use ES6 class inheritance.
	 * ES6 class error is in separate file required at runtime, to avoid errors in Node before v4.
	 */
	var errors = semverSelect(Sequelize.version || '3.0.0', {
		'2.0.0 - 3.x.x': function() {
			// define errors
			var errors = {};

			// general error for all hierarchy errors
			errors.HierarchyError = function(message) {
				Sequelize.Error.call(this, message);
				this.name = 'SequelizeHierarchyError';
			};
			util.inherits(errors.HierarchyError, Sequelize.Error);

			return errors;
		},
		'>=4.0.0-1': function() {
			return require('./errorsEs6')(Sequelize);
		}
	})();

	// alias for error for backward-compatibility
	errors.SequelizeHierarchyError = errors.HierarchyError;

	// add errors to Sequelize and sequelize
	_.extend(Sequelize, errors);
	_.extend(Sequelize.prototype, errors);
};
